import sys                             # לצורך גישה לפרמטרים משורת הפקודה וליציאה בסיום
import os                              # לצורך קבלת PID של התהליך הנוכחי ונתיבי קבצים
import ctypes                          # לצורך בדיקת elevated ולהרצת ShellExecute (UAC)
import win32gui                        # PyWin32: עבודה עם חלונות GUI (HWND)
import win32con                        # PyWin32: קבועים של Windows (WM_CLOSE)
import win32process                    # PyWin32: לקבלת PID מתוך HWND
import psutil                          # לקבלת שם התהליך מתוך PID
import win32api
import time
from PyQt5.QtWidgets import (
    QApplication,
    QWidget,
    QPushButton,
    QVBoxLayout,
    QCheckBox
)
from PyQt5.QtGui import QIcon          # כדי להגדיר את האייקון של החלון
from PyQt5.QtCore import Qt            # ייבוא Qt כדי להשתמש בדגלי יישור וכיוון פריסה

VK_MENU       = 0x12  # מקש ALT
VK_F4         = 0x73  # מקש F4
KEYEVENTF_KEYUP = 0x0002  # לדגל של שחרור מקש (Key Up)


def is_user_admin():
    """
    בודק האם התהליך רץ בהרשאות מנהל (elevated).
    משתמש בפונקציית IsUserAnAdmin מתוך shell32 (ctypes). 
    """
    try:
        return ctypes.windll.shell32.IsUserAnAdmin() != 0
    except Exception:
        return False

def run_as_admin():
    """
    מריץ מחדש את עצמו (אותו קובץ Python) עם הרשאות מנהל (UAC).
    משתמש ב־ShellExecuteW עם הפקודה "runas" כדי לבקש Elevated. 
    """
    python_exe = sys.executable
    script      = os.path.abspath(sys.argv[0])
    params      = " ".join([f'"{arg}"' for arg in sys.argv[1:]])
    ctypes.windll.shell32.ShellExecuteW(
        None,
        "runas",
        python_exe,
        f'"{script}" {params}',
        None,
        1
    )
    sys.exit(0)

def enum_window_callback(hwnd, windows_info):
    """
    callback ל־EnumWindows: עבור כל hwnd הנראה ובעל כותרת,
    שולף PID ושם התהליך (proc_name) ומוסיף לטבלה את הטופל (hwnd, pid, proc_name).
    """
    if win32gui.IsWindowVisible(hwnd) and win32gui.GetWindowText(hwnd):
        try:
            # מקבל PID מתוך HWND 
            _, pid = win32process.GetWindowThreadProcessId(hwnd)
            proc   = psutil.Process(pid)
            proc_name = proc.name()
        except Exception:
            proc_name = ""
        windows_info.append((hwnd, pid, proc_name))

def close_windows_headless(exclude_explorer, log_errors=False, log_path=None):
    """
    גרסה headless (ללא GUI) – סוגר את כל החלונות באופן גרייספול (בלי שליחת Ctrl+S),
    אם exclude_explorer=True → נדלג על חלונות של explorer.exe.
    במידה ו-log_errors=True וב־log_path מצוין נתיב – שומר שגיאות לקובץ לוג.
    לאחר שליחת ההודעות לכל החלונות – סוגר את הסקריפט עצמו.
    """
    windows_info = []
    errors = []  # לאגור שגיאות במקרה ונרשום לוג
    win32gui.EnumWindows(enum_window_callback, windows_info)

    for hwnd, pid, proc_name in windows_info:
        if exclude_explorer and proc_name.lower() == "explorer.exe":
            continue
        if proc_name.lower() == "explorer.exe":
            exp = proc_name.lower() == "explorer.exe"
            try:
                win32gui.PostMessage(hwnd, win32con.WM_CLOSE, 0, 0)  # 
            except Exception as e:
                errors.append(f"לא ניתן לשלוח WM_CLOSE ל־hwnd={hwnd} (pid={pid}, name={proc_name}): {e}")

            continue

        try:
            win32gui.PostMessage(hwnd, win32con.WM_CLOSE, 0, 0)  # 
        except Exception as e:
            errors.append(f"לא ניתן לשלוח WM_CLOSE ל־hwnd={hwnd} (pid={pid}, name={proc_name}): {e}")

    # במידה והוגדרנו לוג ונתיב הקובץ, כותבים לשם את כל השגיאות
    if log_errors and log_path:
        try:
            with open(log_path, "a", encoding="utf-8") as f:
                for line in errors:
                    f.write(line + "\n")
        except Exception:
            # אם לא מצליחים לכתוב ללוג, אין מה לעשות יותר מדי (לא נדפיס)
            pass

    sys.exit(0)

def launch_gui(exclude_explorer, log_errors=False, log_path=None):
    """
    פותח חלון GUI עם תיבת סימון ל־'דלג על סגירת חלונות Explorer'
    וכפתור לסגירה גרייספולית של כל החלונות לפי הבחירה.
    במידה ו־log_errors=True ו־log_path מצוין, גם GUI יכתוב שגיאות לקובץ.
    מגדיר אייקון לחלון (ולשורת המשימות) עם הקובץ 'app_icon.ico'.
    מוודא שהסקריפט רץ elevated לפני שמציג את ה־GUI.
    """

    if not is_user_admin():
        run_as_admin()

    class MainWindow(QWidget):
        def __init__(self):
            super().__init__()
            # קביעת כותרת וגודל החלון
            self.setWindowTitle("סגירת כל היישומים")
            # מגדירים גדלים מספיקים כדי לא להצטופף
            self.setGeometry(100, 100, 500, 200)

            # הגדרת יישור החלון מימין לשמאל
            # זה יגרום לכך שכל הפריטים (QCheckBox, QPushButton) יסודרו מימין לשמאל
            self.setLayoutDirection(Qt.RightToLeft)

            # קביעת אייקון החלון (שורת המשימות + כותרת)
            if getattr(sys, 'frozen', False):
                base_path = sys._MEIPASS         # אם זה EXE, נשתמש ב-_MEIPASS
            else:
                base_path = os.path.dirname(os.path.abspath(__file__))  # אם זה .py, תיקיית הסקריפט
            icon_path = os.path.join(base_path, "app_icon.ico")
            self.setWindowIcon(QIcon(icon_path))

            # תיבת סימון להחרגת חלונות Explorer
            self.chk_exclude_explorer = QCheckBox("דלג על סגירת חלונות Explorer", self)
            self.chk_exclude_explorer.setChecked(True)  # כברירת מחדל נדלג על Explorer

            # כפתור שמפעיל את פעולת הסגירה (גרייספול ללא שמירה)
            self.btn_close_all = QPushButton("סגור את כל החלונות הפתוחים", self)
            self.btn_close_all.clicked.connect(self.handle_close)

            # סידור הרכיבים ב־layout אנכי
            layout = QVBoxLayout()
            # אפשר גם להוסיף יישור פולשני עבור כל וויידג'ט בנפרד, למשל:
            # layout.addWidget(self.chk_exclude_explorer, alignment=Qt.AlignRight)
            # אבל מאחר שהגדרנו את layoutDirection ל־RightToLeft, מספיק להוסיף בלי דגל.
            layout.addWidget(self.chk_exclude_explorer)
            layout.addWidget(self.btn_close_all)
            
            self.setLayout(layout)

            # מאחסנים משתנים לשימוש ב־handle_close
            self.log_errors = log_errors
            self.log_path = log_path

        def handle_close(self):
            """
            מאזין ללחיצה על הכפתור:
            - אוסף את החלונות הפעילים
            - מנסה לשלוח WM_CLOSE לכל חלון (לפי exclude_explorer)
            - אוסף שגיאות לרשימה
            - אם הוגדר log, כותב אותן לקובץ
            - סוגר את התוכנית
            """
            errors = []
            windows_info = []
            win32gui.EnumWindows(enum_window_callback, windows_info)

            for hwnd, pid, proc_name in windows_info:
                if self.chk_exclude_explorer.isChecked() and proc_name.lower() == "explorer.exe":
                    continue

                if proc_name.lower() == "taskmgr.exe":
                    try:
                        win32gui.PostMessage(hwnd, win32con.WM_CLOSE, 0, 0)
                    except Exception:
                        errors.append("אין הרשאה לסגור את Task Manager; יש להריץ את הסקריפט כמנהל.")
                    continue

                try:
                    win32gui.PostMessage(hwnd, win32con.WM_CLOSE, 0, 0)  # 
                except Exception as e:
                    errors.append(f"לא ניתן לשלוח WM_CLOSE ל־hwnd={hwnd} (pid={pid}, name={proc_name}): {e}")

            # אם נתבקשו לרשום לוג, נכין את הקובץ ונכתוב את השגיאות
            if self.log_errors and self.log_path:
                try:
                    with open(self.log_path, "a", encoding="utf-8") as f:
                        for line in errors:
                            f.write(line + "\n")
                except Exception:
                    # במידה ולא מצליחים לרשום, לא מציגים כלום למשתמש
                    pass

            # אחרי הכול נסגור את היישום
            QApplication.quit()

    # אתחול PyQt5 והצגת החלון
    app = QApplication(sys.argv)
    # הגדרת אייקון גלובלי ל־QApplication (לשורת המשימות לפני יצירת החלון)
    app.setWindowIcon(QIcon(os.path.join(
        getattr(sys, 'frozen', False) and sys._MEIPASS or os.path.dirname(os.path.abspath(__file__)),
        "app_icon.ico"
    )))
    window = MainWindow()
    window.show()
    sys.exit(app.exec_())

def main():
    """
    נקודת כניסה ראשית:
    - תומך בפרמטרים:
        - "-e" או "-E" → לסגור גם Explorer
        - "-ne" או "-NE" → לא לסגור Explorer
        - "-l" או "-L" → לכתוב שגיאות לקובץ לוג בשם close_apps.log
    - אם אין פרמטרים חוץ מ־-l → מריץ GUI עם הלוג
    - אם יש פרמטרים הכוללים -e/-ne → מריץ headless בהתאם
    """
    args = [arg.lower() for arg in sys.argv[1:]]  # נעשה lowercase כדי לא לחשוב על רישיות
    exclude_explorer = True  # כברירת מחדל – לא לסגור Explorer
    log_errors = False
    log_path = None

    # אם קיים "-l" בפרמטרים – נדלג על GUI/חלק מהלוגיקה? 
    # קריטריון: אם יש "-e" או "-ne" אז נעבור ל headless, אחרת GUI
    if "-e" in args and "-ne" not in args:
        exclude_explorer = False
    elif "-ne" in args and "-e" not in args:
        exclude_explorer = True

    # בדיקה אם יש "-l" (לוג)
    if "-l" in args:
        log_errors = True
        # נתיב קובץ הלוג בספריית ההפעלה או הה־_MEIPASS
        if getattr(sys, 'frozen', False):
            base_path = sys._MEIPASS
        else:
            base_path = os.path.dirname(os.path.abspath(__file__))
        log_path = os.path.join(base_path, "close_apps.log")

    # החלטה אם נריץ GUI או headless:
    # אם קיים "-e" או "-ne" (ולא GUI בלבד), זה headless; אחרת GUI
    if ("-e" in args) or ("-ne" in args):
        # מצב headless
        if not is_user_admin():
            run_as_admin()
        close_windows_headless(exclude_explorer, log_errors=log_errors, log_path=log_path)
    else:
        # אין פרמטר של -e/-ne, כלומר GUI (גם אם יש רק -l)
        launch_gui(exclude_explorer=exclude_explorer, log_errors=log_errors, log_path=log_path)

if __name__ == "__main__":
    main()
